#include <arch/irq.h>

void irqtab_init()
{
	*((__u16 *)IRQ_VECTOR)=irq0;
}	
 
/*
 *	IRQ and IRQ return paths for Linux Z80
 */

int need_resched;
 
#asm
*	On entry IP is all we can trust
*
*	There are three possible cases to cope with
*
*	kernel_mode=1
*		Interrupted kernel mode code.
*		No task switch allowed
*		Running on a kernel process stack anyway.
*
*	other
*		Interrupted user mode code
*		Switch to kernel stack for process (will be free)
*		Task switch allowed
*
*	bios_mode=1
*		BIOS or other 'strange' code.
*		Must be called from kernel space, but kernel stack is in use
*		Switch to int_stack
*		No task switch allowed.
*
*	We do all of this to avoid per process interrupt stacks and 
*	related nonsense. This way we need only one dedicted int stack
*
*
	extern	_irq0
	
_irq0:
*
*	Save AX and load it with the IRQ number
*
	PUSH	AF
	PUSH	BC
	PUSH	DE
	PUSH	HL
	PUSH	IX
	PUSH 	IY
	EXX
	PUSH	BC
	PUSH	DE
	PUSH	HL
*
*	Set up task switch controller
*
	XOR	A
	LD	HL, kernel_mode
	ADD	A,(HL)
	LD	HL, bios_mode
	ADD	A,(HL)
	PUSH	AF
	
	CALL	_do_IRQ		* Do the work

	POP	AF
	OR	A
	JR	NZ,nosched
	LD	A,(need_resched)
	CALL	NZ,schedule
*
*	If we swapped tasks thats fine. The older user stack is now on
*	ramdisk safely, it will get put back before this context returns
*
	POP	HL
	POP	DE
	POP	BC
	EXX
	POP	IY
	POP	IX
	POP	HL
	POP	DE
	POP	BC
	POP	AF
	IRET
#endasm
