#include <linuxmt/types.h>
#include <linuxmt/sched.h>

/*
 *	save_regs() saves the callers registers and state then returns to
 *	the caller. It in effect freezes a copy of the caller context but doesnt
 *	prevent it being used temporarily beyond that as we do. 
 *
 *	load_regs() restores the callers context and returns skipping out of 
 *	schedule() [our the faked setup] back to the right place. 
 *
 *	fake_save_regs builds a stack frame that returns a new task to a
 *	kernel address of our choice using its own stack/context.
 *
 */

#asm
	EXTERN save_regs
	
save_regs:
*
*	Save the relevant CPU registers. On the Z80 we use the process
*	are for this
*
	PUSH	IX
	PUSH	IY
	LD	IX,(current)
	LD	HL,0
	ADD	HL,SP			* Z80 has no LD HL,SP
	LD	(IX+0),L		* Store SP in the context
	LD	(IX+1),H
	RET
*
*	Unlike save_regs this doesn't return to the caller but
*	to the callers caller.
*	
	EXTERN _load_regs

load_regs:
	POP	BC			* Where to return
	LD	HL,(current)
	LD	E,(HL)			
	INC	HL
	LD	D,(HL)
	EX	DE,HL			* HL is now stack we want
	LD	SP,HL
	POP	IY			* Things we saved
	POP	IX
	PUSH	BC			* Return address of load_regs()
	RET				* Return to caller in context.
	
*
*	System Call Vector
*
	EXTERN syscall
*
syscall:
	PUSH	AF
	LD	A,1
	LD	(kernel_mode),A
	POP	AF
	CALL	syscall			* One stack on Linux/Z80
	PUSH	AF
	XOR	A,A
	LD	(kernel_mode),A
	POP	AF
	RET

	EXTERN fake_save_regs
*
*	int used=fake_save_regs(sp,addr);
*
*	Build a fake return stack in kernel space so that
*	we can have a new task start at a chosen kernel 
*	function while on its kernel stack. We push the
*	registers suitably for
*
_fake_save_regs:
	POP	BC		* Return address
	POP	HL		* addr argument
	POP	IX		* SP argument
	LD	(IX+0), L	* Return
	LD	(IX+1), H	* Address
	PUSH	IX
	PUSH	HL
	PUSH	BC
	LD	HL, 6		* 4 random bytes to load IX/IY with + return
	RET
#endasm

/*
 *	Make task t fork into kernel space. We are in kernel mode
 *	so we fork onto our kernel stack.
 */
 
void kfork_proc(t, addr)
struct task_struct *t;
char *addr;
{
	t->t_regs.sp=t->t_stack;	/* This is a user space ptr */;
	t->t_regs.sp-=fake_save_regs(t->t_regs.sp);
	t->state=TASK_UNINTERRUPTIBLE;
	t->pid=0;
	t->ppid=0;
	t->uid=0;
	t->euid=0;
	t->suid=0;
	t->gid=0;
	t->egid=0;
	t->sgid=0;
	t->session=0;
	t->ppid=0;
	t->t_priority=10;
	t->t_count=0;
	wake_up_process(t);
	schedule();
}
